<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">
<html>
<head>
<style type="text/css">
@media screen, print {
    body {
    font-family: {{ standardFamilyFont }} ! important;
    font-size: {{ mediumFontSize}}px ! important;
    color: #31363b ! important;
    background: #fcfcfc ! important;
    }
    a {
    color: #2980b9 ! important;
    }
    .headerbox {
    background: #eff0f1 ! important;
    color: #31363b ! important;
    border:1px solid #000;
    margin-bottom: 10pt;
    font-size: 22px;
    }
    .headertitle a:link {
    color: rgb(100,100,100) ! important;
    text-decoration: none ! important;
    }
    .headertitle a:visited {
    color: rgb(100,100,100) ! important;
    text-decoration: none ! important;
    }
    .headertitle a:hover{
    color: rgb(100,100,100) ! important;
    text-decoration: none ! important;
    }
    .headertitle a:active {
    color: rgb(100,100,100) ! important;
    text-decoration: none ! important;
    }
    .headertitle {
    padding:2px;
    color: rgb(100,100,100) ! important;
    font-weight: bold;
    text-decoration: none ! important;
    }
    .header {
    color: rgb(120, 125, 129) ! important;
    font-weight: bold;
    padding:2px;
    margin-right: 5px;
    text-decoration: none ! important;
    font-size: {{ mediumFontSize}}px;
    }
    .headertext {
    text-decoration: none ! important;
    color: rgb(120, 125, 129) ! important;
    font-size: {{ mediumFontSize}}px;
    }
    .headimage {
    float: right;
    margin-left: 5px;
    max-width: 150px;
    }
    body {
    clear: none;
    }
    .content {
    display: block;
    margin-bottom: 6px;
    }
    .content > P:first-child {
    margin-top: 1px;
    }
    .content > DIV:first-child {
    margin-top: 1px;
    }
    .content > BR:first-child {
    display: none;
    }
}
</style>
<title>.</title>
</head>
<body>
{% if articles %}
{% for article in articles %}

  <p><div class="article">
  {% if article.strippedTitle %}
  <div class="headertitle" dir="{{ applicationDir }}"><a href="{{ article.articleLinkUrl }}">{{ article.strippedTitle|safe }}</a></div>
  {% endif %}

  {%if article.articlePubDate %}
  <span class="header" dir="{{ applicationDir }}">{{ dateI18n }}:</span><span class="headertext">{{ article.articlePubDate }}</span>
  {% endif %}

  {% if article.author %}
  <br/><span class="header" dir="{{ applicationDir }}">{{ authorI18n }}:</span><span class="headertext">{{ article.author|safe }}</span>
  {% endif %}

  {% if article.enclosure %}
  <br/><span class="header" dir="{{ applicationDir }}">{{ enclosureI18n }}:</span><span class="headertext">{{ article.enclosure|safe }}</span>
  {% endif %}
  </div>

  {% if article.imageFeed %}
  {{ article.imageFeed|safe }}
  {% endif %}
  <hr>

  {% if article.content %}
  <div dir="{{ applicationDir }}"><span class="content">{{ article.content|safe }}</span>
  {% endif %}

  {% if article.articleCompleteStoryLink %}
  <p><a class="contentlink" href="{{ article.articleCompleteStoryLink }}">{{ completeStoryI18n }}</a></p>
  {% endif %}
{% endfor %}
{% endif %}

</body>
