<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">
<html>
<head>
<title>.</title>
<style type="text/css">
@media screen, print {
    body {
    font-family: {{ standardFamilyFont }} ! important;
    font-size: {{ mediumFontSize}}px ! important;
    color: #31363b ! important;
    background: #fcfcfc ! important;
    }
    a {
    color: #2980b9 ! important;
    }
    .actiontable {
    width: 100%;
    }
    .actionrowtable {
    display:table-row;
    }
    .theactioncell{
        display: table-cell;
        padding: 0px 2px; /* just some padding, if needed*/
        white-space: pre; /* this will avoid line breaks*/
    }
    .theactionbigcell{
        width: 100%; /* this will shrink other cells */
    }
    .headerbox {
    background: #eff0f1 ! important;
    color: #31363b ! important;
    border:1px solid #000;
    margin-bottom: 10pt;
    font-size: 22px;
    }
    .headertitle a:link {
    color: rgb(100,100,100) ! important;
    text-decoration: none ! important;
    }
    .headertitle a:visited {
    color: rgb(100,100,100) ! important;
    text-decoration: none ! important;
    }
    .headertitle a:hover{
    color: rgb(100,100,100) ! important;
    text-decoration: none ! important;
    }
    .headertitle a:active {
    color: rgb(100,100,100) ! important;
    text-decoration: none ! important;
    }
    .headertitle {
    padding:2px;
    color: rgb(100,100,100) ! important;
    font-weight: bold;
    text-decoration: none ! important;
    }
    .header {
    color: rgb(120, 125, 129) ! important;
    font-weight: bold;
    padding:2px;
    margin-right: 5px;
    text-decoration: none ! important;
    font-size: {{mediumFontSize}}px;
    }
    .headertext {
    text-decoration: none ! important;
    color: rgb(120, 125, 129) ! important;
    font-size: {{ mediumFontSize}}px;
    }
    .headimage, div.theactioncell a img {
    float: right;
    margin-left: 5px;
    max-width: 150px;
    }
    body {
    clear: none;
    }
    .content {
    display: block;
    margin-bottom: 6px;
    }
    .content > P:first-child {
    margin-top: 1px;
    }
    .content > DIV:first-child {
    margin-top: 1px;
    }
    .content > BR:first-child {
    display: none;
    }
}
</style>
</head>
<body>
{% if articles %}
{% for article in articles %}
  <hr>
  <div class="actiontable">
    <div class="actionrowtable">
      {% with article.articleStatus as result %}
          {% ifequal article.Unread result %}
          <div class="theactioncell"><a href="akregatoraction:markAsRead{{ article.actionToken }}">{% icon "mail-mark-read" small _("Mark as Read") %}</a></div>
          {% endifequal %}
          {% ifequal article.Read result %}
          <div class="theactioncell"><a href="akregatoraction:markAsUnRead{{ article.actionToken }}">{% icon "mail-mark-unread" small _("Mark as Unread") %}</a></div>
          {% endifequal %}
      {% endwith %}
      {% if article.important %}
        <div class="theactioncell"><a href="akregatoraction:markAsImportant{{ article.actionToken }}">{% icon "mail-message" small _("Remove Important Status") %}</a></div>
      {% else %}
        <div class="theactioncell"><a href="akregatoraction:markAsImportant{{ article.actionToken }}">{% icon "mail-mark-important" small _("Mark as Important") %}</a></div>
      {% endif %}
      <div class="theactioncell"><a href="akregatoraction:openInExternalBrowser{{ article.actionToken }}">{% icon "window-new" small _("Open Link in External Browser") %}</a></div>
      <div class="theactioncell"><a href="akregatoraction:openInBackgroundTab{{ article.actionToken }}">{% icon "tab-new" small _("Open In Background Tab") %}</a></div>
      <div class="theactioncell"><a href="akregatoraction:sendFileArticle{{ article.actionToken }}">{% icon "mail-message-new" small _("Send File Address") %}</a></div>
      <div class="theactioncell"><a href="akregatoraction:sendUrlArticle{{ article.actionToken }}">{% icon "mail-message-new" small _("Send Link Address") %}</a></div>
      <div class="theactioncell theactionbigcell"><a href="akregatoraction:delete{{ article.actionToken }}">{% icon "edit-delete" small _("Delete Article") %}</a></div>
    <div>
  </div>
  </div>



  <div class="article">
  {% if article.strippedTitle %}
  <div class="headertitle" dir="{{ applicationDir }}"><a href="{{ article.articleLinkUrl }}">{{ article.strippedTitle|safe }}</a></div>
  {% endif %}

  {%if article.articlePubDate %}
  <span class="header" dir="{{ applicationDir }}">{{ dateI18n }}:</span><span class="headertext">{{ article.articlePubDate }}</span>
  {% endif %}

  {% if article.author %}
  <br/><span class="header" dir="{{ applicationDir }}">{{ authorI18n }}:</span><span class="headertext">{{ article.author|safe }}</span>
  {% endif %}

  {% if article.enclosure %}
  <br/><span class="header" dir="{{ applicationDir }}">{{ enclosureI18n }}:</span><span class="headertext">{{ article.enclosure|safe }}</span>
  {% endif %}
  </div>

  {% if article.imageFeed %}
  {{ article.imageFeed|safe }}
  {% endif %}


  <hr>

  {% if article.content %}
  <div dir="{{ applicationDir }}"><span class="content">{{ article.content|safe }}</span>
  {% endif %}

  {% if article.articleCompleteStoryLink %}
  <p><a class="contentlink" href="{{ article.articleCompleteStoryLink }}">{{ completeStoryI18n }}</a></p>
  {% endif %}
  </div>
  </div><p>
{% endfor %}
{% endif %}

</body>
