/*
    This file is part of Akregator.

    Copyright (C) 2004 Mark Kretschmann <kretschmann@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

    As a special exception, permission is given to link this program
    with any edition of Qt, and distribute the resulting executable,
    without including the source code for Qt in the source distribution.
*/

#include "plugin.h"

namespace Akregator {
Plugin::Plugin(QObject *parent, const QVariantList &) : QObject(parent)
{
}

Plugin::~Plugin()
{
}

void
Plugin::addPluginProperty(const QString &key, const QString &value)
{
    m_properties[key.toLower()] = value;
}

void Plugin::initialize()
{
    doInitialize();
}

QString
Plugin::pluginProperty(const QString &key) const
{
    if (m_properties.find(key.toLower()) == m_properties.end()) {
        return QStringLiteral("false");
    }

    return m_properties[key.toLower()];
}

bool
Plugin::hasPluginProperty(const QString &key) const
{
    return m_properties.find(key.toLower()) != m_properties.end();
}

void Plugin::insertGuiClients(KXMLGUIClient *)
{
}

void Plugin::removeGuiClients(KXMLGUIClient *)
{
}
}
