<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="akregator_shell" version="395">
<MenuBar>
  <Menu noMerge="1" name="file"><text>&amp;File</text>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>

  <Menu noMerge="1" name="edit">
    <text>&amp;Edit</text>
    <Merge/>
  </Menu>

  <Menu noMerge="1" name="view">
    <text>&amp;View</text>
    <Merge/>
  </Menu>

  <Menu noMerge="1" name="go">
    <text>&amp;Go</text>
    <Merge/>
  </Menu>
  
  <Menu noMerge="1" name="feed">
    <text>Fee&amp;d</text>
    <Merge/>
  </Menu>

 <Menu noMerge="1" name="article">
  <text>&amp;Article</text>
    <Merge/>
  </Menu>
  
  <Menu noMerge="1" name="settings" >
   <text>&amp;Settings</text>
   <Merge name="StandardToolBarMenuHandler" />
   <DefineGroup append="top_merge" name="top_merge" />
   <Action name="options_show_menubar" group="top_merge" />
   <Separator/>
   <Action name="options_configure_keybinding" group="settings_configure" />
   <Action name="options_configure_notifications" group="settings_configure" />
   <Action name="options_configure_toolbars" group="settings_configure" />
  </Menu>
</MenuBar>

<ToolBar noMerge="1" name="mainToolBar"><text>Main Toolbar</text>
  <Merge/>
  <Action name="stop"/>
</ToolBar>
</gui>
